package com.jiuqi.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

public final class SortedList implements List {
   private List items;
   private Comparator comparator;
   private int[] findStub = new int[1];

   public SortedList() {
      this.items = new ArrayList();
   }

   public SortedList(int initialCapacity) {
      this.items = new ArrayList(initialCapacity);
   }

   public SortedList(Comparator comparator) {
      this.items = new ArrayList();
      this.comparator = comparator;
   }

   public SortedList(int initalCapacity, Comparator comparator) {
      this.items = new ArrayList(initalCapacity);
      this.comparator = comparator;
   }

   public int compareObjects(Object o1, Object o2) {
      if (this.comparator != null) {
         return this.comparator.compare(o1, o2);
      } else if (o1 instanceof Comparable) {
         return ((Comparable)o1).compareTo(o2);
      } else if (o2 instanceof Comparable) {
         return -((Comparable)o2).compareTo(o1);
      } else {
         throw new UnsupportedOperationException("对象比较时无法转化为Comparable接口！");
      }
   }

   public boolean add(Object o) {
      int[] index = new int[1];
      this.find(o, index);
      this.items.add(index[0], o);
      return true;
   }

   public void add(int index, Object element) {
      this.add(element);
   }

   public void _add(int index, Object element) {
      this.items.add(index, element);
   }

   public boolean addAll(Collection c) {
      List newItems = new ArrayList(c);
      this.reset(newItems);
      return true;
   }

   public boolean addAll(int index, Collection c) {
      return this.addAll(c);
   }

   public void clear() {
      this.items.clear();
   }

   public boolean contains(Object o) {
      return this.find(o, new int[1]);
   }

   public boolean containsAll(Collection c) {
      Iterator itr = c.iterator();

      while(itr.hasNext()) {
         if (!this.contains(itr.next())) {
            return false;
         }
      }

      return true;
   }

   public Object get(int index) {
      return this.items.get(index);
   }

   public int indexOf(Object o) {
      int low = 0;
      int high = this.size() - 1;

      while(low <= high) {
         int mid = (low + high) / 2;
         int flag = this.compareObjects(o, this.get(mid));
         if (flag == 0) {
            if (mid == low) {
               return mid;
            }

            high = mid;
         } else if (flag > 0) {
            low = mid + 1;
         } else {
            high = mid - 1;
         }
      }

      return -1;
   }

   public boolean isEmpty() {
      return this.items.isEmpty();
   }

   public Iterator iterator() {
      return this.items.iterator();
   }

   public int lastIndexOf(Object o) {
      int low = 0;
      int high = this.size() - 1;

      while(low <= high) {
         int mid = (low + high + 1) / 2;
         int flag = this.compareObjects(o, this.get(mid));
         if (flag == 0) {
            if (mid == high) {
               return mid;
            }

            low = mid;
         } else if (flag > 0) {
            low = mid + 1;
         } else {
            high = mid - 1;
         }
      }

      return -1;
   }

   public ListIterator listIterator() {
      return this.items.listIterator();
   }

   public ListIterator listIterator(int index) {
      return this.items.listIterator(index);
   }

   public Object remove(int index) {
      return this.items.remove(index);
   }

   public boolean remove(Object o) {
      return this.items.remove(o);
   }

   public boolean removeAll(Collection c) {
      boolean ret = false;
      Iterator itr = c.iterator();

      while(itr.hasNext()) {
         if (this.remove(itr.next())) {
            ret = true;
         }
      }

      return ret;
   }

   public boolean retainAll(Collection c) {
      List list = new SortedList(c.size(), this.comparator);
      list.addAll(c);
      boolean ret = false;

      for(int i = this.size() - 1; i >= 0; --i) {
         if (!list.contains(this.get(i))) {
            this.remove(i);
            ret = true;
         }
      }

      return ret;
   }

   public Object set(int index, Object element) {
      if (this.compareObjects(this.get(index), element) == 0) {
         return this.items.set(index, element);
      } else {
         Object ret = this.items.get(index);
         this.items.remove(index);
         this.add(element);
         return ret;
      }
   }

   public int size() {
      return this.items.size();
   }

   public List subList(int fromIndex, int toIndex) {
      SortedList result = new SortedList(this.comparator);
      result.items = this.items.subList(fromIndex, toIndex);
      return result;
   }

   public Object[] toArray() {
      return this.items.toArray();
   }

   public Object[] toArray(Object[] a) {
      return this.items.toArray(a);
   }

   public Comparator getComparator() {
      return this.comparator;
   }

   public void setComparator(Comparator comparator) {
      this.comparator = comparator;
   }

   public boolean find(Object element, int[] index) {
      int low = 0;
      int high = this.size() - 1;

      while(low <= high) {
         int mid = (low + high) / 2;
         int flag = this.compareObjects(element, this.get(mid));
         if (flag == 0) {
            index[0] = mid;
            return true;
         }

         if (flag > 0) {
            low = mid + 1;
         } else {
            high = mid - 1;
         }
      }

      index[0] = low;
      return false;
   }

   public int find(Object element) {
      return this.find(element, this.findStub) ? this.findStub[0] : -1;
   }

   public void reset(List items) {
      if (items == null) {
         throw new NullPointerException("items");
      } else {
         Collections.sort(items, new Comparator() {
            public int compare(Object o1, Object o2) {
               return SortedList.this.compareObjects(o1, o2);
            }
         });
         this.items = items;
      }
   }
}
